function [ output_args ] = IPF28_2017_EW_Noiseremoval_HV(export_folder, File_path, HV_name)

% function of radiation correction and thermal noise removal of cross-polarized(HV/VH) SAR image
% application for:
%       the thermal noise removal for images processed by IPF versions below 2.90.
%       
% Inputs:
%       DN_val: the Digital Number value of the Sentinel-1 SAR data
%       cali: the calibration vectors that read from the calibration xml files
%       azNoise or gggDS: the azimuth noise vectors that read from the noise xml files
%       rsNoise: the range noise vectors that read from the noise xml files
%
% Outputs: 
%       Sigma_new_HV: Sigma Naught (normalized radar cross section) of the radiometrically calibrated and denoised Sentinel-1 SAR data
%       lat2: The interpolated latitude matrix of the full EW scene
%       lon2: The interpolated latitude matrix of the full EW scene
%
% path: 
%       new_folder: the path for result saving.
%
% files for help:
%       Sentinel-1 Level-1 Detailed Algorithm Definition, Available: https://sentinel.esa.int/documents/247904/1877131/Sentinel-1-Level-1-Detailed-Algorithm-Definition.
%       Sentinel-1 Product Specification, Available: https://sentinel.esa.int/documents/247904/349449/Sentinel-1_Product_ Specification.

%% Acquire the annotation file path
tic
clc;close all;
addpath(genpath(pwd));
Filename=HV_name;
S_filepath=strrep(File_path,'measurement','') ; %cut out 'string',add 'string'
S_filename=strrep(Filename,'.tiff','.');
tiff_path=[File_path, '/', Filename];

xmln=[S_filepath,'annotation\',S_filename,'xml'];%hv product annotation file
xml_c=[S_filepath,'annotation\calibration\calibration-',S_filename,'xml']; %hv calibration annotation file
xml_n=[S_filepath,'annotation\calibration\noise-',S_filename,'xml'];%hv noise annotation file
xml_m=[S_filepath,'manifest.safe'];%hv manifest.safe file

try
    DN_val=double(imread(tiff_path));%read image (.Tiff)
    DN_val(DN_val==0)=nan;%%Assign the border to be the value of "nan"
    [xf1,yf1]=size(DN_val);
    xmlDoc = xml_read(xmln);%read hv product annotation file
    xml_cc=xml_read(xml_c); %read hv calibration annotation file
    xml_nn=xml_read(xml_n);%read hv noise annotation file
    xmlMani = xml_read(xml_m);%read safe file
    for ii=1:length(xmlMani.metadataSection.metadataObject) %read the IPF version information
        if strcmp(xmlMani.metadataSection.metadataObject(ii).ATTRIBUTE.ID, 'processing')  
            IPFbanben=xmlMani.metadataSection.metadataObject(ii).metadataWrap.xmlData.safe_COLON_processing.safe_COLON_resource.processing.facility.software.ATTRIBUTE.version;
            AUX_CAL=xmlMani.metadataSection.metadataObject(ii).metadataWrap.xmlData.safe_COLON_processing.safe_COLON_resource.processing.resource(2).ATTRIBUTE.name;
            AUX_CALname=AUX_CAL(max(find(AUX_CAL=='/'))+1:end); %the name of needed auxiliary calibration file (aux_cal) 
            xmlauxcal=['AUX_CAL','\',AUX_CALname,'\data\',S_filename(1:4),'aux-cal.xml']; %the aux_cal files need to be downloaded from the website: https://qc.sentinel1.copernicus.eu/
            xml_AUX_CAL=xml_read(xmlauxcal);
            break;
        end
    end
catch
    disp('SAR safe file reading with error.');
    return;
end

%% ------Reading and Interpolation of longitude and latitude matrix to the full image-----------------
%steps:1)read the longitude and latitude information in the tie-points;
%      2)use the MATALB function "griddata()" for interpolation.
clear('lat','lon','lat2','lon2','rowline','colpixel');
n=1;m=1;
lat=zeros;lon=zeros;rowline=zeros;colpixel=zeros; 
ll=length(xmlDoc.geolocationGrid.geolocationGridPointList.geolocationGridPoint);
for ii=1:ll
    lat(n,m)=xmlDoc.geolocationGrid.geolocationGridPointList.geolocationGridPoint(ii).latitude;
    lon(n,m)=xmlDoc.geolocationGrid.geolocationGridPointList.geolocationGridPoint(ii).longitude;
    rowline(n,m)=xmlDoc.geolocationGrid.geolocationGridPointList.geolocationGridPoint(ii).line+1;
    colpixel(n,m)=xmlDoc.geolocationGrid.geolocationGridPointList.geolocationGridPoint(ii).pixel+1;
    if ii<ll
        if rowline(n,m)~=xmlDoc.geolocationGrid.geolocationGridPointList.geolocationGridPoint(ii+1).line+1%change the zero start to be one start
            n=n+1;
            m=1;
        else %if it is still in the same line
            m=m+1;
        end
    end
end

%if the longitude is across 180 degree
if max(max(lon))>0 && min(min(lon))<0 && abs(max(max(lon))-180)<5 && abs(min(min(lon))-(-180)<5)
    lon=rem(360+lon,360);% change its range to be [0,360] degree.
%     lon2=griddata(rowline,colpixel,lon,nX,nY,'cubic');%interpolation
% else  %if the longitude is not across 180 degree,direct interpolation                    
%     lon2=griddata(rowline,colpixel,lon,nX,nY,'cubic');  %first interpolation
% %     lon2=rem(360+lon2,360); %then convert the longitude to the range [0,360]. Also can not convert.
end



%% Interpolation of the Longitude matrix, noting the range of Longitude.
nxx=1:1:xf1;nyy=1:1:yf1;
[nY,nX]=meshgrid(nyy,nxx);
lat2=griddata(rowline,colpixel,lat,nX,nY,'cubic');%interpolation
lon2=griddata(rowline,colpixel,lon,nX,nY,'cubic');%interpolation


%% --------- Interpolation of calibration vector ---------------------------------------
leng=length(xml_cc.calibrationVectorList.calibrationVector);
azimuthXc=zeros();azimuthYc=zeros();cali11=zeros();
for ii=1:leng
    azimuthXcc=xml_cc.calibrationVectorList.calibrationVector(ii).line;   
    azimuthYcc=xml_cc.calibrationVectorList.calibrationVector(ii).pixel;
    azimuthXc(ii,1:azimuthYcc.ATTRIBUTE.count)=azimuthXcc+1; %read the pixel position, and change it to be "starting from 1".
    azimuthYc(ii,1:azimuthYcc.ATTRIBUTE.count)=azimuthYcc.CONTENT+1;
    calitemp=xml_cc.calibrationVectorList.calibrationVector(ii).sigmaNought;   
    cali11(ii,1:azimuthYcc.ATTRIBUTE.count)=calitemp.CONTENT;
end
[rows,cols]=size(cali11);
[cx,cy]=meshgrid(1:1:rows,1:1:cols);
[cxx,cyy]=meshgrid(1:((rows-1)/(xf1-1)):rows,1:((cols-1)/(yf1-1)):cols);
cali1=interp2(cx,cy,cali11',cxx,cyy,'cubic');% interpolation
cali=cali1';
clear('azimuthXcc','azimuthYcc','azimuthXc','azimuthYc','calitemp','cali11','cx','cy','cxx','cyy','cali1'); 


%---------Interpolation of range noise vector,i.e. the noise equivalent sigma naught(NESZ)
%---------1)read the range noise vectors-----------------------------------------------------
leng=length(xml_nn.noiseVectorList.noiseVector);
RangeXn=zeros();RangeYn=zeros();sigma0n1=zeros();
for ii=1:leng
    RangeXn(ii)=xml_nn.noiseVectorList.noiseVector(ii).line+1;%read the pixel position, and change it to be "starting from 1".
    RangeYnn=xml_nn.noiseVectorList.noiseVector(ii).pixel;
    RangeYn(ii,1:RangeYnn.ATTRIBUTE.count)=RangeYnn.CONTENT+1;
    sigma011=xml_nn.noiseVectorList.noiseVector(ii).noiseLut;   
    sigma0n1(ii,1:RangeYnn.ATTRIBUTE.count)=sigma011.CONTENT;
end 
%--------2)process the noise vectors around borders------------------------------------------
clear('x','y');
[x,y]=find(sigma0n1==0);   
if isempty(y)==0 %if there are zero values
    disp('here1');
    for i=2:length(y) %process the value 0 in the noise array
        disp('here2');
        if y(i)~=y(i-1) %The same column
            disp('here3');
            sigma0n1oneline=sigma0n1(:,y(i-1));
            index=find(sigma0n1oneline==0);%find the index of the value 0
            sigma0n1oneline(index)=[];%delete the value 0
            xx=RangeXn;
            xx2=xx(index);
            xx(index)=[];
            if length(sigma0n1oneline)<=10
                disp('the border');
            else
                sigma0n1(index,y(i-1))=interp1(xx,sigma0n1oneline,xx2,'pchip');
                disp(sigma0n1(index,y(i-1)));
            end
        end
    end
end
%---------3)Interpolation to full scene-----------------------------------------------------------
% due to the swathmerging of Sentinel-1 EW data is "Varying Optimal Range Cut Line",thus,the range  noise vector is not given evenly spaced, specially in the inter-swath boundaries. Therefore,interpolation of the range noise vectors is conducted accordingly for each scene
%-------------------------------------------------------------------------------------------------

rsNoise=zeros();
sWnum=length(xmlDoc.imageAnnotation.processingInformation.inputDimensionsList.inputDimensions);
for count=1:sWnum %ÿsuswathڵcompletenoise
    leng=length(xmlDoc.swathMerging.swathMergeList.swathMerge(count).swathBoundsList.swathBounds);%leng>1ʱswathmegingVarying Optimal Range Cut Lineleng=1ʱswathmegingConstant Optimal Range Cut Line
    for ii=1:leng
        clear('firstAzimuthLine','firstRangeSample','lastAzimuthLine','lastRangeSample');
        firstRangeSample=xmlDoc.swathMerging.swathMergeList.swathMerge(count).swathBoundsList.swathBounds(ii).firstRangeSample+1;
        lastRangeSample=xmlDoc.swathMerging.swathMergeList.swathMerge(count).swathBoundsList.swathBounds(ii).lastRangeSample+1;
        firstAzimuthLine=xmlDoc.swathMerging.swathMergeList.swathMerge(count).swathBoundsList.swathBounds(ii).firstAzimuthLine+1;
        lastAzimuthLine=xmlDoc.swathMerging.swathMergeList.swathMerge(count).swathBoundsList.swathBounds(ii).lastAzimuthLine+1;
       
        clear('x','x1','x2','index1','index2','index11','index22','index3','index4');
        [x1,index11]=sort(abs(RangeXn-firstAzimuthLine));%ҳcalisuswathĵһλķλߣindex(1)Ǹߵ±
        if RangeXn(index11(1))<=firstAzimuthLine
            index1=index11(1);
        else
            index1=index11(1)-1;%ʹķλȴsubswath
        end
        [x2,index22]=sort(abs(RangeXn-lastAzimuthLine));% ҳcalisuswathһλķλߣindex(1)Ǹߵ±
        if RangeXn(index22(1))<lastAzimuthLine
            index2=index22(1)+1;
        else
            index2=index22(1);%ʹķλȴsubswath
        end
        if index2-index1<=1 %subswathķλСһֱľʸһάֵsubswathλϵľʸ
            if x1(1)<x2(1)
                midAzsubsw=index11(1);%ȡķλ
            else
                midAzsubsw=index22(1);
            end
            clear('index3','index4','temp','rrsam');
            [x,index3]=sort(abs(RangeYn(midAzsubsw,:)-firstRangeSample));%firstRangeSamplelastRangeSampleод˵
            [x,index4]=sort(abs(RangeYn(midAzsubsw,:)-lastRangeSample));%subswathλСһʱEW߽ϣдλ
            rrsam=firstRangeSample:1:lastRangeSample;
            temp=interp1(RangeYn(midAzsubsw,index3(1):index4(1)),sigma0n1(midAzsubsw,index3(1):index4(1)),rrsam,'pchip');
            for jj=firstAzimuthLine:lastAzimuthLine
                rsNoise(jj,firstRangeSample:lastRangeSample)=temp;
            end
        elseif index2-index1==2  %subswathķλֻһֱʸһάֵsubswathλϵľʸ
            clear('index3','index4');
            midAzsubsw=ceil((index1+index2)/2);%ȡsubswathindex2-1ķλҾ߽λ
            index3=max(find(RangeYn(midAzsubsw,:)>0 & RangeYn(midAzsubsw,:)<=firstRangeSample)); %firstRangeSamplelastRangeSampleод˵
            index4=max(find(RangeYn(midAzsubsw,:)>0 & RangeYn(midAzsubsw,:)<=lastRangeSample));
            
            nx=RangeXn(index1:index2);ny=RangeYn(midAzsubsw,index3:index4);
            [nY,nX]=meshgrid(ny,nx);
            nxx=firstAzimuthLine:1:lastAzimuthLine;nyy=firstRangeSample:1:lastRangeSample;
            [nYY,nXX]=meshgrid(nyy,nxx);
            rsNoise(firstAzimuthLine:lastAzimuthLine,firstRangeSample:lastRangeSample)=griddata(nX,nY,sigma0n1(index1:index2,index3:index4),nXX,nYY,'cubic');%ֵ
            
        else
            clear('index3','index4','temp','sigma0n2','nx','ny','nxx','nyy','nX','nY','nXX','nYY');
            midAzsubsw=ceil((index1+index2)/2);%ȡsubswathindex2-1ķλҾ߽λ
            index3=max(find(RangeYn(midAzsubsw,:)>0 & RangeYn(midAzsubsw,:)<=firstRangeSample));%firstRangeSamplelastRangeSampleод˵
            index4=find(RangeYn(midAzsubsw,:)>0 & RangeYn(midAzsubsw,:)<=lastRangeSample, 1, 'last' );
            sigma0n2=zeros(size(sigma0n1(index1:index2,index3:index4)));
            for mm=index3:index4
                nx=zeros(1,length(RangeXn(index1:index2)));
                nx(1,1)=firstAzimuthLine;
                nx(1,2:end-1)=RangeXn(index1+1:index2-1);%ֻȡ԰subswathڵķλ
                nx(1,end)=lastAzimuthLine;
                temp=interp1(nx(1,2:end-1),sigma0n1(index1+1:index2-1,mm)',nx,'pchip');%plot(temp);
                sigma0n2(:,mm-index3+1)=temp';
            end
            ny=RangeYn(midAzsubsw,index3:index4);
            [nY,nX]=meshgrid(ny,nx);
            nxx=firstAzimuthLine:1:lastAzimuthLine;nyy=RangeYn(midAzsubsw,index3):RangeYn(midAzsubsw,index4);
            [nYY,nXX]=meshgrid(nyy,nxx);
            rsNoise(firstAzimuthLine:lastAzimuthLine,RangeYn(midAzsubsw,index3):RangeYn(midAzsubsw,index4))=griddata(nX,nY,sigma0n2,nXX,nYY,'cubic');%ֵ
        end
    end
end
clearvars -EXCEPT tiff_path xmlDoc xml_AUX_CAL export_folder S_filepath IPFbanben DN_val cali rsNoise lon2 lat2

%% -------------------------------------------------
% Calculation of the descalloping gain.The variables include: naz,faz,ta,nburst,tfb,tovl,tb,tovr,tburst,ka,ks,kt,vs,f,radarWavelen,steerAngle,gAAEP,gds,gDS.
% Refer to the paper of Park et al.(2018), and the one by Sun and Li (2020).

%------------------calculate the focused burst azimuth time (tburst)---------------------- --------
sWnum=length(xmlDoc.imageAnnotation.processingInformation.inputDimensionsList.inputDimensions);%the number of sub-swaths
naz=zeros(sWnum);
for count=1:sWnum
    naz(count)=xmlDoc.imageAnnotation.processingInformation.inputDimensionsList.inputDimensions(count).numberOfInputLines;%naz,used to calculate the number of azimuth lines in Tfb.
end
faz=xmlDoc.imageAnnotation.imageInformation.azimuthFrequency;%faz,used to calculate the frequency in Tfb.
aPnum=length(xmlDoc.antennaPattern.antennaPatternList.antennaPattern);
firstlinetimeorig=xmlDoc.imageAnnotation.imageInformation.productFirstLineUtcTime;%use the first azimuth time to judge if the image spans 0 o'clock
firstlinetime=str2double(firstlinetimeorig(12:13))*3600+str2double(firstlinetimeorig(15:16))*60+str2double(firstlinetimeorig(18:end));%transfer the "hours-minutes-seconds" to be "seconds"
lastlinetimeorig=xmlDoc.imageAnnotation.imageInformation.productLastLineUtcTime;
if strcmp(lastlinetimeorig(6:7),firstlinetimeorig(6:7))==1 && strcmp(lastlinetimeorig(9:10),firstlinetimeorig(9:10))==1 %to judge the image spans 0 o'clock or not
    lastlinetime=str2double(lastlinetimeorig(12:13))*3600+str2double(lastlinetimeorig(15:16))*60+str2double(lastlinetimeorig(18:end));%transfer the "hours-minutes-seconds" to be "seconds"
else
    lastlinetime=(str2double(lastlinetimeorig(12:13))+24)*3600+str2double(lastlinetimeorig(15:16))*60+str2double(lastlinetimeorig(18:end));%if it passes the 0 o'clock, adding 24 hours to the time.
end
m=1;ta=zeros();
for ii=1:aPnum  %ii=1; 
   ewwhich=xmlDoc.antennaPattern.antennaPatternList.antennaPattern(ii).swath;
   n=str2double(ewwhich(3));%the number of sub-swath (EW)
   temp=xmlDoc.antennaPattern.antennaPatternList.antennaPattern(ii).azimuthTime;%ta,azimuth time of first burst line,the time is recorded as "2018-08-02T20:16:17.440192".
   if strcmp(temp(6:7),firstlinetimeorig(6:7))==1 && strcmp(temp(9:10),firstlinetimeorig(9:10))==1%to judge the image spans 0 o'clock or not
       ta(n,m)=str2double(temp(12:13))*3600+str2double(temp(15:16))*60+str2double(temp(18:end));
   else
       ta(n,m)=(str2double(temp(12:13))+24)*3600+str2double(temp(15:16))*60+str2double(temp(18:end));
   end
   if rem(ii,sWnum)==0 %read the azimuth time in sequence, n means the sub-swath, m means the burst.
       m=m+1;
       disp(ii);
   end
end
clear('aPnum','firstlinetimeorig','lastlinetimeorig','ewwhich','n','temp');

nburstinput=length(xmlDoc.generalAnnotation.azimuthFmRateList.azimuthFmRate);%The burst number of the input image, that is, the burst number before it is processed as GRD data.
nburst=zeros();
tc=zeros();tb=zeros();tovl=zeros();
for n=1:sWnum   %calculate the tb,tc,tovr in each burst in each sub-swath.
    nburst(n)=length(find(ta(n,:)~=0));%nburst(n) is the burst number when the image was processed as GRD data. used to calculate ta,tb,tburst...
    %linesofbu=(ta(n,3)-ta(n,2))*faz;
    tfb=naz(n)/((nburstinput)*faz);%the period before burst merging. 
    for m=2:nburst(n)
        tc(n,m)=ta(n,m-1)+tfb/2.0;%in the paper of Park(2018),the function for tc is wrong.
        tovl(n,m-1)=tfb-(ta(n,m)-ta(n,m-1));
        tb(n,m)=ta(n,m)+tovl(n,m-1)/2.0;  
        %tovr(n,m)=tb(n,m)-tc(n,m);
        if tb(n,m)>lastlinetime %for the last burst
            tb(n,m)=lastlinetime;
        end
        
    end
    tb(n,1)=ta(n,1)+tovl(n,1)/2.0;
    %tb(n,1)=tb(n,2)-(ta(n,2)-ta(n,1));
    tc(n,1)=tb(n,1)-(ta(n,2)-ta(n,1))/2.0;
    tc(n,nburst(n)+1)=ta(n,nburst(n))+tfb/2.0;
    tb(n,nburst(n)+1)=lastlinetime;
end

%-------calculate tburst,boundary-------------------------------------------------------------------
%------------------read vs,f, and calculate ks,ka,kt------------------------------------------------
steerRate=xmlDoc.generalAnnotation.productInformation.azimuthSteeringRate;
radarfre=xmlDoc.generalAnnotation.productInformation.radarFrequency;
radarWavelen=299792458.0/radarfre;%calculate the radar wave length
leng=length(xmlDoc.generalAnnotation.orbitList.orbit);
vs=zeros();azTime=zeros();
for ii=1:leng
    temptime=xmlDoc.generalAnnotation.orbitList.orbit(ii).time;
    azTime(ii)=str2double(temptime(12:13))*3600+str2double(temptime(15:16))*60+str2double(temptime(18:end));%time transfer
    vsx=xmlDoc.generalAnnotation.orbitList.orbit(ii).velocity.x;%use the satellite's velocity in the middle time to represent the whole azimuth.
    vsy=xmlDoc.generalAnnotation.orbitList.orbit(ii).velocity.y;
    vsz=xmlDoc.generalAnnotation.orbitList.orbit(ii).velocity.z;
    vs(ii)=sqrt(vsx^2+vsy^2+vsz^2);%the satellite's velocity
end
[x1,index11]=sort(abs(azTime-firstlinetime));%find the nearest calibration sample to the first azimuth line.
if azTime(index11(1))<=firstlinetime
    index1=index11(1);
else
    index1=index11(1)-1;%make the width of samples larger than the width of the subswath.
end
[x2,index22]=sort(abs(azTime-lastlinetime));% %find the nearest calibration sample to the last azimuth line.
if azTime(index22(1))<lastlinetime
    index2=index22(1)+1;
else
    index2=index22(1);%make the width of samples larger than the width of the subswath.
end
clear('x1','index11','x2','index22','temptime','vsx','vsy','vsz');
vs=mean(vs(index1:index2));
ks=-2*vs*(steerRate*pi/180.0)/radarWavelen;%calculate ks, note that,the unit of steerRate needs to be changed from degree/s to rad/s.

%--------------calculate the slantRangetime at the middle of the sub-swath----------------------------
tSR=zeros();
for ii=1:sWnum  
    ewwhich=xmlDoc.antennaPattern.antennaPatternList.antennaPattern(ii).swath;
    n=str2double(ewwhich(3));
    slantRangetime=xmlDoc.antennaPattern.antennaPatternList.antennaPattern(ii).slantRangeTime;%read the slant time in each sub-swath
    num=length(slantRangetime.CONTENT); 
    tSR(n)=slantRangetime.CONTENT(floor(num/2));%get the slantRangetime at the middle of the sub-swath along the range direction.
    disp(n);disp(num);
end
ka=zeros();
leng=length(xmlDoc.generalAnnotation.azimuthFmRateList.azimuthFmRate);
mm=1;
for nn=1:leng
    clear azTime;
    azTime=xmlDoc.generalAnnotation.azimuthFmRateList.azimuthFmRate(nn).azimuthTime;
    azTime=str2double(azTime(12:13))*3600+str2double(azTime(15:16))*60+str2double(azTime(18:end));%time transfer
    if leng==nburst(count) %if the number of azimuthFmRateList is equal to the number of nburst
        t0=xmlDoc.generalAnnotation.azimuthFmRateList.azimuthFmRate(nn).t0; %read the coefficients of function azimuthFMrate
        try
            c=xmlDoc.generalAnnotation.azimuthFmRateList.azimuthFmRate(nn).azimuthFmRatePolynomial;
            for count=1:sWnum
                ka(count,mm)=c.CONTENT(1)+c.CONTENT(2)*(tSR(count)-t0)+c.CONTENT(3)*(tSR(count)-t0)^2;
            end
        catch
            c0=xmlDoc.generalAnnotation.azimuthFmRateList.azimuthFmRate(nn).c0;
            c1=xmlDoc.generalAnnotation.azimuthFmRateList.azimuthFmRate(nn).c1;
            c2=xmlDoc.generalAnnotation.azimuthFmRateList.azimuthFmRate(nn).c2;
            for count=1:sWnum
                ka(count,mm)=c0+c1*(tSR(count)-t0)+c2*(tSR(count)-t0)^2;
            end
        end
        mm=mm+1;
    elseif leng<nburst(count) %if the number of azimuthFmRateList is less than the number of nburst,add one value in the array ka, making ka(1)=ka(2).
        if nn==1
            mm=2;
        end
        t0=xmlDoc.generalAnnotation.azimuthFmRateList.azimuthFmRate(nn).t0;
        try
            c=xmlDoc.generalAnnotation.azimuthFmRateList.azimuthFmRate(nn).azimuthFmRatePolynomial;
            for count=1:sWnum
                ka(count,mm)=c.CONTENT(1)+c.CONTENT(2)*(tSR(count)-t0)+c.CONTENT(3)*(tSR(count)-t0)^2;
                ka(count,1)=ka(count,2);
            end
        catch
            c0=xmlDoc.generalAnnotation.azimuthFmRateList.azimuthFmRate(nn).c0;
            c1=xmlDoc.generalAnnotation.azimuthFmRateList.azimuthFmRate(nn).c1;
            c2=xmlDoc.generalAnnotation.azimuthFmRateList.azimuthFmRate(nn).c2;
            for count=1:sWnum
                ka(count,mm)=c0+c1*(tSR(count)-t0)+c2*(tSR(count)-t0)^2;
                ka(count,1)=ka(count,2);
            end
        end
        mm=mm+1;
    elseif azTime>=firstlinetime %if the number of azimuthFmRateList is larger than the number of nburst,select the ka with azTime larger than the first azimuth line.
        t0=xmlDoc.generalAnnotation.azimuthFmRateList.azimuthFmRate(nn).t0; 
        try
            c=xmlDoc.generalAnnotation.azimuthFmRateList.azimuthFmRate(nn).azimuthFmRatePolynomial;
            for count=1:sWnum
                ka(count,mm)=c.CONTENT(1)+c.CONTENT(2)*(tSR(count)-t0)+c.CONTENT(3)*(tSR(count)-t0)^2;%tSR(count)is the mid-SlantRangetime of every sub-swath
            end
        catch
            c0=xmlDoc.generalAnnotation.azimuthFmRateList.azimuthFmRate(nn).c0;
            c1=xmlDoc.generalAnnotation.azimuthFmRateList.azimuthFmRate(nn).c1;
            c2=xmlDoc.generalAnnotation.azimuthFmRateList.azimuthFmRate(nn).c2;
            for count=1:sWnum
                ka(count,mm)=c0+c1*(tSR(count)-t0)+c2*(tSR(count)-t0)^2;%tSR(count)is the mid-SlantRangetime of every sub-swath
            end
        end
        mm=mm+1;
    end
end
 
kt=-ka.*ks./(ks-ka);%the number of kt is equal to that of nburst
clear('steerRate','radarfre','leng','tSR','mm','azTime','t0','c');
%------------------calculate the steerAngle,(t)-----------------------------------------------
%-------------Refer to the functions in paper of Sun and Li(2020)------------------------------
timeinterval=xmlDoc.imageAnnotation.imageInformation.azimuthTimeInterval;%time interval of azimuth lines,and firstlinetime,lastlinetime have been read before.
totallines=xmlDoc.imageAnnotation.imageInformation.numberOfLines;%note that the image has black borders.
numPerburst=zeros();steerAngle=zeros();tburst=zeros();
for count=1:sWnum
    aztime=firstlinetime;ii=1;
    if nburst(count)<= length(kt(count,:))
        kt(count,nburst(count)+1)=kt(count,nburst(count));
    else
        kt(count,nburst(count))=kt(count,nburst(count)-1);
        kt(count,nburst(count)+1)=kt(count,nburst(count));
    end
    for nn=1:nburst(count)+1
        num=1;
        tburst(count,2*nn-1)=aztime-tc(count,nn);%calculate the focused burst time of the first line in the burst.
        steerAngle(count,ii)=radarWavelen*kt(count,nn)*tburst(count,2*nn-1)/(2.0*vs);%calculate the focused steer angle of the first line in the burst.
        ii=ii+1;
        while((aztime-tb(count,nn))<timeinterval/1000.0)% aztime<=tb(count,nn)
            aztime=aztime+timeinterval;
            num=num+1;%used to record the number of azimuth lines in each burst.
        end
        numPerburst(count,nn)=num-1;%the number of azimuth lines in each burst.
        tburst(count,2*nn)=aztime-timeinterval-tc(count,nn);%calculate the focused burst time of the last line in the burst.
        steerAngle(count,ii)=radarWavelen*kt(count,nn)*tburst(count,2*nn)/(2.0*vs);%calculate the focused steer angle of the last line in the burst.
        ii=ii+1;
    end
    disp(ii-1);disp(aztime);
end

s1=sum(numPerburst,2);%to make sure if the number of total lines is equal to the sum of every burst. 
steerAngle=steerAngle.*180.0./pi;%convert the unit to be degree
clear('aztime','s1');
%---------read the LUT of gAAEP and steerAngle-------------------------------------------------------
valuesAAEP=zeros();nn=1;
for ii=1:length(xml_AUX_CAL.calibrationParamsList.calibrationParams)
    swath=xml_AUX_CAL.calibrationParamsList.calibrationParams(ii).swath;
    polarisation=xml_AUX_CAL.calibrationParamsList.calibrationParams(ii).polarisation;
    if strcmp(swath(1:2),'EW')==1 && strcmp(polarisation,'HV')==1  %judge the polarization and imaging mode, it can be changed for other modes.%***********attention*************************
        azAngleIncre=xml_AUX_CAL.calibrationParamsList.calibrationParams(ii).azimuthAntennaElementPattern.azimuthAngleIncrement;
        temp=xml_AUX_CAL.calibrationParamsList.calibrationParams(ii).azimuthAntennaElementPattern.values;
        leng=length(temp.CONTENT);
        valuesAAEP(nn,1:leng)=temp.CONTENT;
        nn=nn+1; %nn is the number of sub-swath
        if(swath(3)==num2str(sWnum))
            break;
        end
    end
end
steerAnglefix=zeros(size(steerAngle));
for count=1:sWnum
    for nn=1:nburst(count)+1
        steerAnglefix(count,2*nn-1)=fix(steerAngle(count,2*nn-1)/azAngleIncre)-1;%the first azimuth lines of bursts shift forward by 1,
        steerAnglefix(count,2*nn)=fix(steerAngle(count,2*nn)/azAngleIncre)+1;%the last azimuth lines of bursts shift backward by 1. To make sure the samples totally surround the lines.
    end
end
clear('swath','polarisation','temp','leng');

%------------get gAAEP of every line in the burst by interpolation------------------------------------
azlineAngle=zeros(sWnum,totallines);gAAEP=zeros(sWnum,totallines);
for count=1:sWnum
    numAAEP=length(valuesAAEP(count,:));%the length of valuesAAEP is odd, such as 201. the middle value is 0, corresponding to tburst=0,angle=0,AAEP=0. forward, the angle is negative;backward, the angle is positive.
    midnum=(numAAEP+1)/2;%the middle position
    num=1;
    for nn=1:nburst(count)+1 
        clear ('X','Y');Y=zeros();
        angle1=steerAnglefix(count,2*nn-1)*azAngleIncre;%the first azimuth angle of the burst,the unit is degree.
        angle2=steerAnglefix(count,2*nn)*azAngleIncre;%the last azimuth angle of the burst,the unit is degree.
        X=angle1:azAngleIncre:angle2;
        index1=midnum + steerAnglefix(count,2*nn-1);%the index of first azimuth angle of the burst
        index2=midnum + steerAnglefix(count,2*nn); %the index of last azimuth angle of the burst
        for ii=index1:index2
            Y(ii-index1+1)=valuesAAEP(count,ii);%the LUT values of the AAEP in every burst.
        end

        if numPerburst(count,nn)==0 %for the last burst
            break;
        else
            xi=linspace(steerAngle(count,2*nn-1),steerAngle(count,2*nn),numPerburst(count,nn));%every azimuth line
            yi=interp1(X, Y, xi,'linear');
            azlineAngle(count,num:num+numPerburst(count,nn)-1)=xi;
            gAAEP(count,num:num+numPerburst(count,nn)-1)=yi;%the unit is dB
            num=num+numPerburst(count,nn);
        end
    end
end
gds=power(10,gAAEP/20); %Transfer the unit dB to be linear sigma0, based on the file "Sentinel-1 Level-1 Detailed Algorithm Definition".
gDS=gds.^(-1); %the descalloping gain !!
clear('numAAEP','midnum','angle1','angle2','index1','index2','X','Y','xi','yi','num','count');
clearvars -EXCEPT tiff_path xmlDoc export_folder S_filepath IPFbanben DN_val cali gDS rsNoise lon2 lat2

%% -----Read the "scene boundaries" in each subswath(EW1 to EW5).---------------------------------------
%Due to the swathmerging of Sentinel-1 EW data is "Varying Optimal Range Cut Line".Thus,the scaling and balancing of NESZ need to be done in each scene,to avoid inaccurate noise modifying in the inter-swath boundaries.
%-----------------------------------------------------------------------------------------------------
%% read the boundaries of each scene in each subswath(EW1 to EW5)
sWnum=length(xmlDoc.imageAnnotation.processingInformation.inputDimensionsList.inputDimensions);%the number of subswath. 5 for EW mode, 3 for IW mode.
for count=1:sWnum
    leng(count)=length(xmlDoc.swathMerging.swathMergeList.swathMerge(count).swathBoundsList.swathBounds);%leng is the number of scenes in each subswath.
end %if leng>1, the swathmeging is "Varying Optimal Range Cut Line"; if leng=1, the swathmeging is "Constant Optimal Range Cut Line".
clear max;
ll=max(leng);
clear('firstAzimuthLine','firstRangeSample','lastAzimuthLine','lastRangeSample');
firstRangeSample=zeros;lastRangeSample=zeros;
firstAzimuthLine=zeros;lastAzimuthLine=zeros;
for count=1:sWnum 
    leng=length(xmlDoc.swathMerging.swathMergeList.swathMerge(count).swathBoundsList.swathBounds);
    for ii=1:leng
        firstRangeSample(count,ii)=xmlDoc.swathMerging.swathMergeList.swathMerge(count).swathBoundsList.swathBounds(ii).firstRangeSample;
        lastRangeSample(count,ii)=xmlDoc.swathMerging.swathMergeList.swathMerge(count).swathBoundsList.swathBounds(ii).lastRangeSample;
        firstAzimuthLine(count,ii)=xmlDoc.swathMerging.swathMergeList.swathMerge(count).swathBoundsList.swathBounds(ii).firstAzimuthLine;
        lastAzimuthLine(count,ii)=xmlDoc.swathMerging.swathMergeList.swathMerge(count).swathBoundsList.swathBounds(ii).lastAzimuthLine;
    end   
    firstRangeSample(count,leng+1:ll)=nan;
    lastRangeSample(count,leng+1:ll)=nan;
    firstAzimuthLine(count,leng+1:ll)=nan;
    lastAzimuthLine(count,leng+1:ll)=nan;
end
firstRangeSample=firstRangeSample+1; %change the pixels of the array to be starting from 1.
lastRangeSample=lastRangeSample+1; %starting from 1.
firstAzimuthLine=firstAzimuthLine+1; %starting from 1.
lastAzimuthLine=lastAzimuthLine+1; %starting from 1.

%% extend the descalloping gain calculated in each sub-swath to the entire image
gggDS=zeros(size(DN_val));
for count=1:sWnum
    leng=length(xmlDoc.swathMerging.swathMergeList.swathMerge(count).swathBoundsList.swathBounds);
	% leng>1swathmeging is Varying Optimal Range Cut Line
	% leng=1swathmeging is Constant Optimal Range Cut Line
    for ii=1:leng
        clear rswide;
        rswide=lastRangeSample(count,ii)-firstRangeSample(count,ii)+1;%the range width of the subswath
        nyy=firstRangeSample(count,ii):lastRangeSample(count,ii);%the range index of the subswath
        nxx=firstAzimuthLine(count,ii):lastAzimuthLine(count,ii);
        %nnn1=gDS(count,nxx);plot(nnn1);
        gggDS1=repmat(gDS(count,nxx),rswide,1);
        gggDS(nxx,nyy)=gggDS1';
    end
end
clear('rswide');
clearvars -EXCEPT tiff_path xmlDoc export_folder S_filepath IPFbanben DN_val cali gggDS rsNoise lon2 lat2 firstRangeSample lastRangeSample firstAzimuthLine lastAzimuthLine
%------------------------------------------------------------------------------------------------------
%% --------remove the black borders---------------------------------------------
% 1)find the position of the black borders
DN_val(find(isnan(DN_val)))=0;
[xf1,yf1]=size(DN_val);
for ii=1:yf1
    id=find(DN_val(:,ii)<=20);
    if length(id)/xf1>0.15
        idy=ii;
    else
        idy=ii;
        break;
    end
end
for ii=1:xf1
    id=find(DN_val(ii,:)<=20);
    if length(id)/yf1>0.15
        idx=ii;
    else        
        idx=ii;
        break;
    end
end
for ii=xf1:-1:1
    id=find(DN_val(ii,:)<=20);
    if length(id)/yf1>0.15
        idxh=ii;
    else
        idxh=ii;
        break;      
    end
end
for ii=yf1:-1:1
    id=find(DN_val(:,ii)<=20);
    if length(id)/xf1>0.15
        idyh=ii;
    else
        idyh=ii;
        break;
    end
end 
% %-----------------------------------------------------------------------------------------------------
% %------------2)remove the black borders for the data arrays------------------------------------------
DN_val(1:idx,:)=nan; DN_val(idxh:end, :)=nan; DN_val(:, 1:idy)=nan; DN_val(:, idyh:end)=nan; 
cali(1:idx,:)=nan; cali(idxh:end, :)=nan; cali(:, 1:idy)=nan; cali(:, idyh:end)=nan; 
rsNoise(1:idx,:)=nan; rsNoise(idxh:end, :)=nan; rsNoise(:, 1:idy)=nan; rsNoise(:, idyh:end)=nan; 
gggDS(1:idx,:)=nan; gggDS(idxh:end, :)=nan; gggDS(:, 1:idy)=nan; gggDS(:, idyh:end)=nan; 


clearvars -EXCEPT idx idxh idy idyh tiff_path export_folder S_filepath IPFbanben DN_val cali gggDS rsNoise lon2 lat2 firstRangeSample lastRangeSample firstAzimuthLine lastAzimuthLine

%% Land Mask based on GSHHS and GreenlandNorth_Landmask3.tif
% if this image locates along the coastlines at the north of Greenland, use
% the 'GreenlandNorth_Landmask2.tif' file to do land mask. 
% 'GreenlandNorth_Landmask2.tif' was processed based on the coastlines from EOSDIS worldview at 250m resolution. 
tt1=floor(min(min(lat2))/0.1)*0.1: 0.1 : ceil(max(max(lat2))/0.1)*0.1;
tt2=floor(min(min(lon2))/0.1)*0.1: 0.1 : ceil(max(max(lon2))/0.1)*0.1;
if  (~isempty(intersect(tt1, 78:0.1:84))  &&  ~isempty(intersect(tt2, -82:0.1:-10)) ) ||...
    ( ~isempty(intersect(tt1, 75.5:0.1:80))  &&  ~isempty(intersect(tt2, -23:0.1:-16)) ) 
    % -----Land Mask1----------------------------------
    Mask=imread('coastlines\GreenlandNorth_Landmask3.tif');
    Mask = double(Mask>0);
    figure;imshow(Mask,[-1 1]);
    
    [xf2,yf2]=size(Mask);
    Interval=0.006916302; 
    lon_O=-121.283063472;
    lat_O=89.8475606782;
    lat22=abs(lat2-lat_O);%γת89.433520830792NΪԭ
    lon22=abs(lon2-lon_O);%ת-36.63482462962Ϊԭ
    
    Row_t=round(lat22./Interval);%vfiеĵMaskеص
    Col_t=round(lon22./Interval);%vfiеĵMaskеص
    Row_t(Row_t==0)=1;%0ֵ
    Col_t(Col_t==0)=1;%0ֵ
    
    [xf2,yf2]=size(Mask);
    abIndex=(Col_t-1).*xf2+Row_t;  % 1end*endattention: along the cols
    abIndex=abIndex(:)';
    BW=Mask(abIndex);   
    BW2=reshape(BW,size(lat22));

else
    % -----Land Mask2----------------------------------------------------------
    % Because the backscatter of land ice is extremely higher than the sea ice, the land is masked for better noise scaling.
    % GSHHS.tif is the coastline data in Arctic (extracted from the GSHHG database),and it has been processed to be a 0/1 array.The first pixel in the upper left corner of the array is starting from (90N180E).
    Mask=imread('GSHHS.tif');
    Interval=0.0027777778;%the pixel interval(degree);   
    
    lat22=90-lat2;%change the latitude to be starting at 90N.
    lon22=rem(lon2+180,360);%change the longitude to be starting at 180E.
    
    Row_t=round(lat22./Interval);%Calculate the lat22's coordinates in the Mask.
    Col_t=round(lon22./Interval);
    Row_t(Row_t==0)=1;
    Col_t(Col_t==0)=1;
    
    [xf2,yf2]=size(Mask);
    abIndex=(Col_t-1).*xf2+Row_t;%compute the pixels' absolute coordinates based on the row and col.
    abIndex=abIndex(:)';
    BW=Mask(abIndex); %extract the coastline array.  
    BW2=reshape(BW,size(lat22));
end

BW3=ones(size(DN_val));
BW3(BW2==1)=nan;   % find the land, and make is to be nan. Getting the land mask.
DN_masked=double(DN_val.*BW3);  % Land masked DN value array.
clearvars -EXCEPT idx idxh idy idyh tiff_path export_folder S_filepath IPFbanben DN_val cali gggDS rsNoise lon2 lat2 DN_masked firstRangeSample lastRangeSample firstAzimuthLine lastAzimuthLine


%% **************Calling functions to remove thermal noise *******************************************
% additive noise scaling and balancing, multiplicative noise removal
% gggDS=azNoise; 
ScalingParameter_calculation; %Refer to the corresponding .m file
clearvars -EXCEPT idx idxh idy idyh tiff_path export_folder S_filepath Sigma_SN Sigma_NESZ Sigma_N_ESA lon2 lat2 cali DN_masked firstRangeSample lastRangeSample firstAzimuthLine lastAzimuthLine kns2 imgStyle varss 


%% two ways to do noise vector balancing. Refer to the corresponding .m file
BalancingParameter_calculation;  % 1)
% BalancingParameter_calculation20220904;   % 2)


%% 1) textural noise removal using our method. 
MultiplicativeNoise_crrection; %Refer to the corresponding .m file
Sigma_new_HV=sigma_Deno_corrected; 
denoing_part2= '';

% the user can choose 1) or 2) to make textural noise removal.
%% 2) textural noise removal using the method proposed by (Park, et.al., 2019)
% tic
% tt = strfind(S_filepath, '_EW_');
% [Sigma_new_HV, ~] = Parkmethod02(Sigma_SN, ...
%     sigma_n_balanced, sigma_Deno_balanced, S_filepath(tt-3:tt-1), ...
%     firstRangeSample, lastRangeSample, firstAzimuthLine, lastAzimuthLine );
% toc
% denoing_part2 = '_Park02';


close all
toc
%% ------save the result.----------------------------------
tt = strfind(S_filepath, '_EW_');
Filename = S_filepath(tt-3:tt-3+66);
new_folder=[ export_folder, '/', Filename, '/'];
mkdir(new_folder);


f2=figure; imshow(Sigma_new_HV, [-0.0015 0.007]); 
tt = colormap(gray(300));f2.Colormap= tt(40:end, :);  
colorbar('fontsize',11,'color','k'); 
print (f2,'-dpng','-r150',...
        [new_folder,  Filename(end-3:end),'_HV', denoing_part2, '.png']);

save([new_folder, Filename(end-3:end), '_HV', denoing_part2, '.mat'], ...
             'Sigma_new_HV', 'lon2', 'lat2', 'idx', 'idxh', 'idy', 'idyh');


close all

%--------------------------------------------------------------------------------------------------------------
%------------FINISHED!-----------------------------------------------------------------------------------------
