function [ output_args ] = IPF29_2018_EW_Noiseremoval_HV(export_folder, File_path, HV_name )

%  major contents.
% function of radiation correction and thermal noise removal of cross-polarized(HV/VH) SAR image
% application for:
%       the thermal noise removal for images processed by IPF version 2.9x or above.
%       
% Inputs:
%       DN_val: the Digital Number value of the Sentinel-1 SAR data
%       cali: the calibration vectors that read from the calibration xml files
%       azNoise or gggDS: the azimuth noise vectors that read from the noise xml files
%       rsNoise: the range noise vectors that read from the noise xml files
%
% Outputs: 
%       Sigma_new_HV: Sigma Naught (normalized radar cross section) of the radiometrically calibrated and denoised Sentinel-1 SAR data
%       lat2: The interpolated latitude matrix of the full EW scene
%       lon2: The interpolated latitude matrix of the full EW scene
%
% path: 
%       new_folder: the path for result saving.
%
% files for help:
%       Sentinel-1 Level-1 Detailed Algorithm Definition, Available:
%       https://sentinel.esa.int/documents/247904/1877131/Sentinel-1-Level-1-Detailed-Algorithm-Definition.
%       Sentinel-1 Product Specification, Available: 
%       https://sentinel.esa.int/documents/247904/349449/Sentinel-1_Product_ Specification.
%
%% ---------Acquire the annotation file path-----------------------------------------------------
tic   
clc;close all;
Filename=HV_name;
S_filepath=strrep(File_path,'measurement','') ; %cut out 'string',add 'string'
S_filename=strrep(Filename,'.tiff','.');
tiff_path=[File_path, '/', Filename];

xmln=[S_filepath,'annotation/',S_filename,'xml'];%hv product annotation file
xml_c=[S_filepath,'annotation/calibration/calibration-',S_filename,'xml']; %hv calibration annotation file
xml_n=[S_filepath,'annotation/calibration/noise-',S_filename,'xml'];%hv noise annotation file
xml_m=[S_filepath,'manifest.safe'];%hv manifest.safe file

try
    DN_val=double(imread(tiff_path));%read the tiff image
    DN_val(DN_val==0)=nan;%Assign the border to be nan
    [xf1,yf1]=size(DN_val);
    xmlDoc = xml_read(xmln);%read product annotation file
    xml_cc=xml_read(xml_c); %read calibration annotation file
    xml_nn=xml_read(xml_n);%read noise annotation file
%     xmlMani = xml_read(xml_m);%read safe file
%     for ii=1:length(xmlMani.metadataSection.metadataObject) %read the IPF version information
%         if strcmp(xmlMani.metadataSection.metadataObject(ii).ATTRIBUTE.ID, 'processing')  
%             try
%                 IPFbanben=xmlMani.metadataSection.metadataObject(ii).metadataWrap.xmlData.safe_COLON_processing.safe_COLON_resource.processing.facility.software.ATTRIBUTE.version;  %2021֮ǰ
%             catch
%                 IPFbanben=xmlMani.metadataSection.metadataObject(ii).metadataWrap.xmlData.safe_COLON_processing.safe_COLON_resource.safe_COLON_processing.safe_COLON_facility.safe_COLON_software.ATTRIBUTE.version;  %2021֮
%             end
%             break;
%         end
%     end
catch
    disp('SAR safe file reading with error.');
    return;
end

%% ------Reading and Interpolation of longitude and latitude matrix to the full image-----------------
%steps:1)read the longitude and latitude information in the tie-points;
%      2)use the MATALB function "griddata()" for interpolation.
clear('lat','lon','lat2','lon2','rowline','colpixel');
n=1;m=1;
lat=zeros;lon=zeros;rowline=zeros;colpixel=zeros; 
ll=length(xmlDoc.geolocationGrid.geolocationGridPointList.geolocationGridPoint);
for ii=1:ll
    lat(n,m)=xmlDoc.geolocationGrid.geolocationGridPointList.geolocationGridPoint(ii).latitude;
    lon(n,m)=xmlDoc.geolocationGrid.geolocationGridPointList.geolocationGridPoint(ii).longitude;
    rowline(n,m)=xmlDoc.geolocationGrid.geolocationGridPointList.geolocationGridPoint(ii).line+1;
    colpixel(n,m)=xmlDoc.geolocationGrid.geolocationGridPointList.geolocationGridPoint(ii).pixel+1;
    if ii<ll
        if rowline(n,m)~=xmlDoc.geolocationGrid.geolocationGridPointList.geolocationGridPoint(ii+1).line+1%change the zero start to be one start
            n=n+1;
            m=1;
        else %if it is still in the same line
            m=m+1;
        end
    end
end

%if the longitude is across 180 degree
if max(max(lon))>0 && min(min(lon))<0 && abs(max(max(lon))-180)<5 && abs(min(min(lon))-(-180)<5)
    lon=rem(360+lon,360);% change its range to be [0,360] degree.
%     lon2=griddata(rowline,colpixel,lon,nX,nY,'cubic');%interpolation
% else  %if the longitude is not across 180 degree,direct interpolation                    
%     lon2=griddata(rowline,colpixel,lon,nX,nY,'cubic');  %first interpolation
% %     lon2=rem(360+lon2,360); %then convert the longitude to the range [0,360]. Also can not convert.
end


%% -------Interpolation of the Longitude matrix, noting the range of Longitude.---------------
nxx=1:1:xf1;nyy=1:1:yf1;
[nY,nX]=meshgrid(nyy,nxx);
lat2=griddata(rowline,colpixel,lat,nX,nY,'cubic');%interpolation
lon2=griddata(rowline,colpixel,lon,nX,nY,'cubic');%interpolation


%% ----------Interpolation of calibration vector--------------------------------------------------
leng=length(xml_cc.calibrationVectorList.calibrationVector);
azimuthXc=zeros();azimuthYc=zeros();cali11=zeros();
for ii=1:leng
    azimuthXcc=xml_cc.calibrationVectorList.calibrationVector(ii).line;   
    azimuthYcc=xml_cc.calibrationVectorList.calibrationVector(ii).pixel;
    azimuthXc(ii,1:azimuthYcc.ATTRIBUTE.count)=azimuthXcc+1; %read the pixel position, and change it to be "starting from 1".
    azimuthYc(ii,1:azimuthYcc.ATTRIBUTE.count)=azimuthYcc.CONTENT+1;
    calitemp=xml_cc.calibrationVectorList.calibrationVector(ii).sigmaNought;   
    cali11(ii,1:azimuthYcc.ATTRIBUTE.count)=calitemp.CONTENT;
end
[rows,cols]=size(cali11);
[cx,cy]=meshgrid(1:1:rows,1:1:cols);
[cxx,cyy]=meshgrid(1:((rows-1)/(xf1-1)):rows,1:((cols-1)/(yf1-1)):cols);
cali1=interp2(cx,cy,cali11',cxx,cyy,'cubic');% interpolation
cali=cali1';
clear('azimuthXcc','azimuthYcc','azimuthXc','azimuthYc','calitemp','cali11','cx','cy','cxx','cyy','cali1'); 

%% ------Interpolation of range noise vector,i.e. the noise equivalent sigma naught(NESZ)---------
%---------1)read the range noise vectors------------------------------------------------------------------------
leng=length(xml_nn.noiseRangeVectorList.noiseRangeVector);
RangeXn=zeros();RangeYn=zeros();sigma0n1=zeros();
for ii=1:leng
    RangeXn(ii)=xml_nn.noiseRangeVectorList.noiseRangeVector(ii).line+1;%read the pixel position, and change it to be "starting from 1".
    RangeYnn=xml_nn.noiseRangeVectorList.noiseRangeVector(ii).pixel;
    RangeYn(ii,1:RangeYnn.ATTRIBUTE.count)=RangeYnn.CONTENT+1;
    sigma011=xml_nn.noiseRangeVectorList.noiseRangeVector(ii).noiseRangeLut;   
    sigma0n1(ii,1:RangeYnn.ATTRIBUTE.count)=sigma011.CONTENT;
end 
%------------------------------------------------------------------------------------------
%--------2)process the noise vectors around borders------------------------------------------
clear('x','y');
[x,y]=find(sigma0n1==0);   
if isempty(y)==0 %if there is zero value
    disp('here1');
    for i=2:length(y) %process the value 0 in the noise array
        disp('here2');
        if y(i)~=y(i-1) % The same column
            disp('here3');
            sigma0n1oneline=sigma0n1(:,y(i-1));
            index=find(sigma0n1oneline==0);%find the index of the value 0
            sigma0n1oneline(index)=[];%delete the value 0
            xx=RangeXn;
            xx2=xx(index);
            xx(index)=[];
            if length(sigma0n1oneline)<=10
                disp('the border');
            else
                sigma0n1(index,y(i-1))=interp1(xx,sigma0n1oneline,xx2,'pchip');
                disp(sigma0n1(index,y(i-1)));
            end
        end
    end
end
%---------3)Interpolation to full scene-----------------------------------------------------------
% due to the swathmerging of Sentinel-1 EW data is "Varying Optimal Range Cut Line",thus,the range  noise vector is not given evenly spaced, specially in the inter-swath boundaries. Therefore,interpolation of the range noise vectors is conducted accordingly for each scene
%-------------------------------------------------------------------------------------------------
clear('azNoise','azNoise11');
leng=length(xml_nn.noiseAzimuthVectorList.noiseAzimuthVector);
rsNoise=zeros();
for ii=1:leng   %about swathmerging, please see the "swathBoundsList" part of the "Sentinel-1_Product_Specification" file.
    clear('firstAzimuthLine','firstRangeSample','lastAzimuthLine','lastRangeSample');
    firstAzimuthLine=xml_nn.noiseAzimuthVectorList.noiseAzimuthVector(ii).firstAzimuthLine+1;%read the pixel position, and change it to be "starting from 1".
    firstRangeSample=xml_nn.noiseAzimuthVectorList.noiseAzimuthVector(ii).firstRangeSample+1;%read the pixel position, and change it to be "starting from 1".
    lastAzimuthLine=xml_nn.noiseAzimuthVectorList.noiseAzimuthVector(ii).lastAzimuthLine+1;%read the pixel position, and change it to be "starting from 1".
    lastRangeSample=xml_nn.noiseAzimuthVectorList.noiseAzimuthVector(ii).lastRangeSample+1; %read the pixel position, and change it to be "starting from 1".
    clear('x','x1','x2','index1','index2','index11','index22','index3','index4');
    [x1,index11]=sort(abs(RangeXn-firstAzimuthLine));%find the noise vector closest to the first azimuth line of the scene. 
    if RangeXn(index11(1))<=firstAzimuthLine
        index1=index11(1);
    else
        index1=index11(1)-1;%make the width of noise vectors larger than the scene
    end
    [x2,index22]=sort(abs(RangeXn-lastAzimuthLine));%find the noise vector closest to the last azimuth line of the scene. 
    if RangeXn(index22(1))<lastAzimuthLine
        index2=index22(1)+1;
    else
        index2=index22(1);%make the width of noise vectors larger than the scene
    end
    if index2-index1<=1 %if the width of the scene is less than one grid of noise vectors,
        if x1(1)<x2(1)
            midAzsubsw=index11(1);%directly use the nearest noise vector
        else
            midAzsubsw=index22(1);
        end
        clear('index3','index4','temp','rrsam');
        [x,index3]=sort(abs(RangeYn(midAzsubsw,:)-firstRangeSample));
        [x,index4]=sort(abs(RangeYn(midAzsubsw,:)-lastRangeSample));
        rrsam=firstRangeSample:1:lastRangeSample;
        temp=interp1(RangeYn(midAzsubsw,index3(1):index4(1)),sigma0n1(midAzsubsw,index3(1):index4(1)),rrsam,'pchip');
        for jj=firstAzimuthLine:lastAzimuthLine
            rsNoise(jj,firstRangeSample:lastRangeSample)=temp; 
        end
    elseif index2-index1==2  %if the width of the scene is just larger than one grid of noise vectors,
        clear('index3','index4');
        midAzsubsw=ceil((index1+index2)/2);%use the mid-line to find the range boundaries of this scene
        index3=find(RangeYn(midAzsubsw,:)==firstRangeSample);%the firstRangeSample of the scene
        index4=find(RangeYn(midAzsubsw,:)==lastRangeSample);%the lastRangeSample of the scene
        
        nx=RangeXn(index1:index2);ny=RangeYn(midAzsubsw,index3:index4);
        [nY,nX]=meshgrid(ny,nx);
        nxx=firstAzimuthLine:1:lastAzimuthLine;nyy=firstRangeSample:1:lastRangeSample;
        [nYY,nXX]=meshgrid(nyy,nxx);
        rsNoise(firstAzimuthLine:lastAzimuthLine,firstRangeSample:lastRangeSample)=griddata(nX,nY,sigma0n1(index1:index2,index3:index4),nXX,nYY,'cubic');
    else  %if the width of the scene includes more than one grids of noise vectors,
        clear('index3','index4','temp','sigma0n2','nx','ny','nxx','nyy','nX','nY','nXX','nYY');
        midAzsubsw=ceil((index1+index2)/2);%use the mid-line to find the range boundaries of this scene
        index3=find(RangeYn(midAzsubsw,:)==firstRangeSample);%the firstRangeSample of the scene
        index4=find(RangeYn(midAzsubsw,:)==lastRangeSample);%the lastRangeSample of the scene
        sigma0n2=zeros(size(sigma0n1(index1:index2,index3:index4)));
        for mm=index3:index4
            nx=zeros(1,length(RangeXn(index1:index2)));
            nx(1,1)=firstAzimuthLine;
            nx(1,2:end-1)=RangeXn(index1+1:index2-1);%just use the noise vectors locating in the interior of the scene
            nx(1,end)=lastAzimuthLine;
            temp=interp1(nx(1,2:end-1),sigma0n1(index1+1:index2-1,mm)',nx,'pchip');%plot(temp);
            sigma0n2(:,mm-index3+1)=temp';
        end
        ny=RangeYn(midAzsubsw,index3:index4);
        [nY,nX]=meshgrid(ny,nx);
        nxx=firstAzimuthLine:1:lastAzimuthLine;nyy=firstRangeSample:1:lastRangeSample;
        [nYY,nXX]=meshgrid(nyy,nxx);
        rsNoise(firstAzimuthLine:lastAzimuthLine,firstRangeSample:lastRangeSample)=griddata(nX,nY,sigma0n2,nXX,nYY,'cubic');
    end
end
%--------------------------------------------------------------------------------------------------------------
%% --------Interpolation of azimuth noise vectors,i.e. the descalloping gain-----------------------------------
clear('azNoise','azNoise11');
leng=length(xml_nn.noiseAzimuthVectorList.noiseAzimuthVector);
azNoise=zeros();
for ii=1:leng 
    clear('firstAzimuthLine','firstRangeSample','lastAzimuthLine','lastRangeSample');
    firstAzimuthLine=xml_nn.noiseAzimuthVectorList.noiseAzimuthVector(ii).firstAzimuthLine+1;
    firstRangeSample=xml_nn.noiseAzimuthVectorList.noiseAzimuthVector(ii).firstRangeSample+1;
    lastAzimuthLine=xml_nn.noiseAzimuthVectorList.noiseAzimuthVector(ii).lastAzimuthLine+1;
    lastRangeSample=xml_nn.noiseAzimuthVectorList.noiseAzimuthVector(ii).lastRangeSample+1; 
    
    azLine=xml_nn.noiseAzimuthVectorList.noiseAzimuthVector(ii).line;
    noiseAzimuthLut=xml_nn.noiseAzimuthVectorList.noiseAzimuthVector(ii).noiseAzimuthLut;
    noiseAzimuthLutval=noiseAzimuthLut.CONTENT;
    if ischar(noiseAzimuthLutval)%if the (double) array is very long, the MATLAB will wrongly read it to be type char
        noiseAzimuthLutval2=zeros();
        index1=zeros(1,azLine.ATTRIBUTE.count+1);
        index1(2:end-1)=find(isspace(noiseAzimuthLutval));
        index1(1)=0;index1(end)=length(noiseAzimuthLutval)+1;
        for jj2=1:length(index1)-1
            noiseAzimuthLutval2(jj2)=str2double(noiseAzimuthLutval(index1(jj2)+1:index1(jj2+1)-1));
        end
        noiseAzimuthLutval=noiseAzimuthLutval2;
    end
    azline2=azLine.CONTENT;
    if ischar(azline2)%if the (double) array is very long, the MATLAB will wrongly read it to be type char
        azline2temp=zeros();
        index1=zeros(1,azLine.ATTRIBUTE.count+1);
        index1(2:end-1)=find(isspace(azline2));
        index1(1)=0;index1(end)=length(azline2)+1;
        for jj2=1:length(index1)-1
            azline2temp(jj2)=str2double(azline2(index1(jj2)+1:index1(jj2+1)-1));
        end
        azline2=azline2temp;
    end
    if azLine.ATTRIBUTE.count==1 %if there is only one sample line in the scene, it should be very narrow.
        azNoise(firstAzimuthLine:lastAzimuthLine,firstRangeSample:lastRangeSample)=noiseAzimuthLutval;
    else
        azxx=firstAzimuthLine:1:lastAzimuthLine;
        azNoise11=interp1(azline2,noiseAzimuthLutval,azxx,'pchip');%first, along the azimuth direction, interp the noise vector to be each pixel.
        swathWide=lastRangeSample-firstRangeSample+1;
        azNoise22=repmat(azNoise11,swathWide,1);%then, copy the azimuth noise vector to each row. 
        azNoise(firstAzimuthLine:lastAzimuthLine,firstRangeSample:lastRangeSample)=azNoise22';%transposition
    end
end
clearvars -EXCEPT tiff_path xmlDoc export_folder S_filepath IPFbanben DN_val cali azNoise rsNoise lon2 lat2

%% -----Read the "scene boundaries" in each subswath(EW1 to EW5).---------------------------------------
%Due to the swathmerging of Sentinel-1 EW data is "Varying Optimal Range Cut Line".Thus,the scaling and balancing of NESZ need to be done in each scene,to avoid inaccurate noise modifying in the inter-swath boundaries.
%-----------------------------------------------------------------------------------------------------
sWnum=length(xmlDoc.imageAnnotation.processingInformation.inputDimensionsList.inputDimensions);%the number of subswath. 5 for EW mode, 3 for IW mode.
for count=1:sWnum
    leng(count)=length(xmlDoc.swathMerging.swathMergeList.swathMerge(count).swathBoundsList.swathBounds);%leng is the number of scenes in each subswath.
end %if leng>1, the swathmeging is "Varying Optimal Range Cut Line"; if leng=1, the swathmeging is "Constant Optimal Range Cut Line".
clear max;
ll=max(leng);
clear('firstAzimuthLine','firstRangeSample','lastAzimuthLine','lastRangeSample');
firstRangeSample=zeros;lastRangeSample=zeros;
firstAzimuthLine=zeros;lastAzimuthLine=zeros;
for count=1:sWnum 
    leng=length(xmlDoc.swathMerging.swathMergeList.swathMerge(count).swathBoundsList.swathBounds);
    for ii=1:leng
        firstRangeSample(count,ii)=xmlDoc.swathMerging.swathMergeList.swathMerge(count).swathBoundsList.swathBounds(ii).firstRangeSample;
        lastRangeSample(count,ii)=xmlDoc.swathMerging.swathMergeList.swathMerge(count).swathBoundsList.swathBounds(ii).lastRangeSample;
        firstAzimuthLine(count,ii)=xmlDoc.swathMerging.swathMergeList.swathMerge(count).swathBoundsList.swathBounds(ii).firstAzimuthLine;
        lastAzimuthLine(count,ii)=xmlDoc.swathMerging.swathMergeList.swathMerge(count).swathBoundsList.swathBounds(ii).lastAzimuthLine;
    end   
    firstRangeSample(count,leng+1:ll)=nan;
    lastRangeSample(count,leng+1:ll)=nan;
    firstAzimuthLine(count,leng+1:ll)=nan;
    lastAzimuthLine(count,leng+1:ll)=nan;
end
firstRangeSample=firstRangeSample+1; %change the pixels of the array to be starting from 1.
lastRangeSample=lastRangeSample+1; %starting from 1.
firstAzimuthLine=firstAzimuthLine+1; %starting from 1.
lastAzimuthLine=lastAzimuthLine+1; %starting from 1.
clearvars -EXCEPT tiff_path export_folder S_filepath IPFbanben DN_val cali azNoise rsNoise lon2 lat2 firstRangeSample lastRangeSample firstAzimuthLine lastAzimuthLine
%--------------------------------------------------------------------------------------------------------------
%% --------remove the black borders----------------------------------------------------------------------------
%------------1)find the position of the black borders----------------------------------------------------------
DN_val(find(isnan(DN_val)))=0;
[xf1,yf1]=size(DN_val);
for ii=1:yf1
    id=find(DN_val(:,ii)<=20);
    if length(id)/xf1>0.15
        idy=ii;
    else
        idy=ii;
        break;
    end
end
for ii=1:xf1
    id=find(DN_val(ii,:)<=20);
    if length(id)/yf1>0.15
        idx=ii;
    else        
        idx=ii;
        break;
    end
end
for ii=xf1:-1:1
    id=find(DN_val(ii,:)<=20);
    if length(id)/yf1>0.15
        idxh=ii;
    else
        idxh=ii;
        break;      
    end
end
for ii=yf1:-1:1
    id=find(DN_val(:,ii)<=20);
    if length(id)/xf1>0.15
        idyh=ii;
    else
        idyh=ii;
        break;
    end
end 
% %--------------------------------------------------------------------------------------------------------------
% %------------2)remove the black borders for the data arrays------------------------------------------------------
DN_val(1:idx,:)=nan; DN_val(idxh:end, :)=nan; DN_val(:, 1:idy)=nan; DN_val(:, idyh:end)=nan; 
cali(1:idx,:)=nan; cali(idxh:end, :)=nan; cali(:, 1:idy)=nan; cali(:, idyh:end)=nan; 
rsNoise(1:idx,:)=nan; rsNoise(idxh:end, :)=nan; rsNoise(:, 1:idy)=nan; rsNoise(:, idyh:end)=nan; 
azNoise(1:idx,:)=nan; azNoise(idxh:end, :)=nan; azNoise(:, 1:idy)=nan; azNoise(:, idyh:end)=nan; 


clearvars -EXCEPT idx idxh idy idyh tiff_path export_folder S_filepath IPFbanben DN_val cali azNoise rsNoise lon2 lat2 firstRangeSample lastRangeSample firstAzimuthLine lastAzimuthLine
%--------------------------------------------------------------------------------------------------------------
%% Land Mask based on GSHHS and GreenlandNorth_Landmask3.tif
% if this image locates along the coastlines at the north of Greenland, use
% the 'GreenlandNorth_Landmask2.tif' file to do land mask. 
% 'GreenlandNorth_Landmask2.tif' was processed based on the coastlines from EOSDIS worldview at 250m resolution. 
tt1=floor(min(min(lat2))/0.1)*0.1: 0.1 : ceil(max(max(lat2))/0.1)*0.1;
tt2=floor(min(min(lon2))/0.1)*0.1: 0.1 : ceil(max(max(lon2))/0.1)*0.1;
if  (~isempty(intersect(tt1, 78:0.1:84))  &&  ~isempty(intersect(tt2, -82:0.1:-10)) ) ||...
    ( ~isempty(intersect(tt1, 75.5:0.1:80))  &&  ~isempty(intersect(tt2, -23:0.1:-16)) ) 
    % -----Land Mask1----------------------------------
    Mask = imread('coastlines\GreenlandNorth_Landmask3.tif');
    Mask = double(Mask>0);
    figure;imshow(Mask,[-1 1]);
    
    [xf2,yf2]=size(Mask);
    Interval=0.006916302; 
    lon_O=-121.283063472;
    lat_O=89.8475606782;
    lat22=abs(lat2-lat_O);%γת89.433520830792NΪԭ
    lon22=abs(lon2-lon_O);%ת-36.63482462962Ϊԭ
    
    Row_t=round(lat22./Interval);%vfiеĵMaskеص
    Col_t=round(lon22./Interval);%vfiеĵMaskеص
    Row_t(Row_t==0)=1;%0ֵ
    Col_t(Col_t==0)=1;%0ֵ
    
    [xf2,yf2]=size(Mask);
    abIndex=(Col_t-1).*xf2+Row_t;
    abIndex=abIndex(:)';
    BW=Mask(abIndex);   
    BW2=reshape(BW,size(lat22));

else
    % -----Land Mask2---------------------------------------------------------
    % Because the backscatter of land ice is extremely higher than the sea ice, the land is masked for better noise scaling.
    % GSHHS.tif is the coastline data in Arctic (extracted from the GSHHG database),and it has been processed to be a 0/1 array.The first pixel in the upper left corner of the array is starting from (90N180E).
    Mask=imread('GSHHS.tif'); 
    Interval=0.0027777778;%the pixel interval(degree);   
    
    lat22=90-lat2;%change the latitude to be starting at 90N.
    lon22=rem(lon2+180,360);%change the longitude to be starting at 180E.
    
    Row_t=round(lat22./Interval);%Calculate the lat22's coordinates in the Mask.
    Col_t=round(lon22./Interval);
    Row_t(Row_t==0)=1;
    Col_t(Col_t==0)=1;
    
    [xf2,yf2]=size(Mask);
    abIndex=(Col_t-1).*xf2+Row_t;%compute the pixels' absolute coordinates based on the row and col.
    abIndex=abIndex(:)';
    BW=Mask(abIndex); %extract the coastline array.  
    BW2=reshape(BW,size(lat22));
end
    
BW3=ones(size(DN_val)); 
BW3(BW2==1)=nan;   % find the land, and make it to be nan. Getting the land mask.
DN_masked=double(DN_val.*BW3);  % Land masked DN value array.
% clear Mask lat22 lon22 Row_t Col_t abIndex BW BW2 BW3 
clearvars -EXCEPT idx idxh idy idyh tiff_path export_folder S_filepath IPFbanben DN_val cali azNoise rsNoise lon2 lat2 DN_masked firstRangeSample lastRangeSample firstAzimuthLine lastAzimuthLine



%% --------Calling functions to remove thermal noise---------------------------------------------
%-----------including: additive noise scaling and balancing, multiplicative noise removal----------------------
gggDS=azNoise;  %Unified name, the azNoise provided by the ESA is rightly the descalloping gain in AAEP.ScalingParameter_calculation; 
ScalingParameter_calculation;  %Refer to the corresponding .m file
clearvars -EXCEPT idx idxh idy idyh tiff_path export_folder S_filepath Sigma_SN Sigma_NESZ Sigma_N_ESA lon2 lat2 cali DN_masked firstRangeSample lastRangeSample firstAzimuthLine lastAzimuthLine kns2 imgStyle varss 


%% two ways to do noise vector balancing. Refer to the corresponding .m file
BalancingParameter_calculation;  % 1)
% BalancingParameter_calculation20220904;   % 2)


%% 1) textural noise removal using our method. 
MultiplicativeNoise_crrection; %Refer to the corresponding .m file
Sigma_new_HV=sigma_Deno_corrected; 
denoing_part2= '';


% the user can choose 1) or 2) to make textural noise removal.
%% 2) textural noise removal using the method proposed by (Park, et.al., 2019)
% tic
% tt = strfind(S_filepath, '_EW_');
% [Sigma_new_HV, ~] = Parkmethod02(Sigma_SN, ...
%     sigma_n_balanced, sigma_Deno_balanced, S_filepath(tt-3:tt-1), ...
%     firstRangeSample, lastRangeSample, firstAzimuthLine, lastAzimuthLine );
% toc
% denoing_part2 = '_Park02';


close all
toc
%% ------save the result.----------------------------------
tt = strfind(S_filepath, '_EW_');
Filename = S_filepath(tt-3:tt-3+66);
new_folder=[ export_folder, '/', Filename, '/'];
mkdir(new_folder);


f2=figure; imshow(Sigma_new_HV, [-0.0015 0.007]); 
tt = colormap(gray(300));f2.Colormap= tt(40:end, :);  
colorbar('fontsize',11,'color','k'); 
print (f2,'-dpng','-r150',...
        [new_folder,  Filename(end-3:end),'_HV', denoing_part2, '.png']);

save([new_folder, Filename(end-3:end), '_HV', denoing_part2, '.mat'], ...
             'Sigma_new_HV', 'lon2', 'lat2', 'idx', 'idxh', 'idy', 'idyh');


close all

%--------------------------------------------------------------------------------------------------------------
%------------FINISHED!-----------------------------------------------------------------------------------------
